package dev.struchkov.godfather.quarkus.core;

import dev.struchkov.godfather.main.domain.EventContainer;
import dev.struchkov.godfather.quarkus.context.service.EventDispatching;
import dev.struchkov.godfather.quarkus.context.service.EventHandler;
import io.smallrye.mutiny.Multi;
import io.smallrye.mutiny.Uni;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

public class EventDispatchingImpl implements EventDispatching {

    private final Map<Class, List<EventHandler>> eventHandlerMap;

    public EventDispatchingImpl(List<EventHandler> eventProviders) {
        this.eventHandlerMap = eventProviders.stream().collect(Collectors.groupingBy(EventHandler::getEventClass));
    }

    @Override
    public Uni<Void> dispatch(EventContainer event) {
        if (checkNotNull(event)) {
            return Uni.createFrom().voidItem()
                    .onItem().transformToUni(
                            v -> {
                                // запросы к боту из чатов: https://core.telegram.org/bots/inline
                                final Optional<List<EventHandler>> optHandlers = getHandler(event.getType());
                                if (optHandlers.isPresent()) {
                                    return Multi.createFrom().iterable(optHandlers.get())
                                            .onItem().transformToUni(
                                                    eventHandler -> eventHandler.handle(event)
                                            ).concatenate().collect().asList().replaceWithVoid();
                                }
                                return Uni.createFrom().voidItem();
                            }
                    );
        }
        return Uni.createFrom().voidItem();
    }

    private Optional<List<EventHandler>> getHandler(Class type) {
        return Optional.ofNullable(eventHandlerMap.get(type));
    }

}
