package dev.struchkov.godfather.quarkus.core.action;

import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.quarkus.context.service.Sending;
import dev.struchkov.godfather.quarkus.domain.BoxAnswer;
import dev.struchkov.godfather.quarkus.domain.unit.AnswerCheck;
import dev.struchkov.godfather.quarkus.domain.unit.MainUnit;
import dev.struchkov.godfather.quarkus.domain.unit.UnitRequest;
import io.smallrye.mutiny.Uni;

import java.util.Objects;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;
import static dev.struchkov.haiti.utils.Checker.checkTrue;

/**
 * Обработчик Unit-а {@link AnswerCheck}.
 *
 * @author upagge [11/07/2019]
 */
public class AnswerCheckAction implements ActionUnit<AnswerCheck<Message>, Message> {

    private final Sending sending;

    public AnswerCheckAction(Sending sending) {
        this.sending = sending;
    }

    @Override
    public Uni<UnitRequest<MainUnit, Message>> action(UnitRequest<AnswerCheck<Message>, Message> unitRequest) {
        final AnswerCheck<Message> unit = unitRequest.getUnit();
        final Message message = unitRequest.getMessage();

        return unit.getCheck().checked(message)
                .onItem().call(checkValue -> {
                    if (checkTrue(checkValue)) {
                        final BoxAnswer answerIfTrue = unit.getIntermediateAnswerIfTrue();
                        if (checkNotNull(answerIfTrue)) {
                            answerIfTrue.setRecipientIfNull(message.getFromPersonId());
                            return sending.send(answerIfTrue);
                        }
                    } else {
                        final BoxAnswer answerIfFalse = unit.getIntermediateAnswerIfFalse();
                        if (checkNotNull(answerIfFalse)) {
                            answerIfFalse.setRecipientIfNull(message.getFromPersonId());
                            return sending.send(answerIfFalse);
                        }
                    }
                    return Uni.createFrom().voidItem();
                })
                .onItem().transform(
                        checkValue -> {
                            if (checkTrue(checkValue)) {
                                return UnitRequest.of(Objects.requireNonNullElse(unit.getUnitTrue(), unit), message);
                            } else {
                                return UnitRequest.of(Objects.requireNonNullElse(unit.getUnitFalse(), unit), message);
                            }
                        }
                );
    }

    @Override
    public String getUnitType() {
        return AnswerCheck.TYPE;
    }

    @Override
    public Class<Message> getMessageType() {
        return Message.class;
    }

}
