package dev.struchkov.godfather.quarkus.core.action;

import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.quarkus.domain.unit.AnswerSave;
import dev.struchkov.godfather.quarkus.domain.unit.MainUnit;
import dev.struchkov.godfather.quarkus.domain.unit.UnitRequest;
import dev.struchkov.godfather.quarkus.domain.unit.func.CheckSave;
import dev.struchkov.godfather.quarkus.domain.unit.func.PreservableData;
import dev.struchkov.godfather.quarkus.domain.unit.func.Pusher;
import dev.struchkov.godfather.quarkus.domain.unit.func.preser.AnswerSavePreservable;
import io.smallrye.mutiny.Uni;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerSave}.
 *
 * @author upagge [11/07/2019]
 */
public class AnswerSaveAction<D> implements ActionUnit<AnswerSave<Message, D>, Message> {

    @Override
    public Uni<UnitRequest<MainUnit, Message>> action(UnitRequest<AnswerSave<Message, D>, Message> unitRequest) {
        final AnswerSave<Message, D> answerSave = unitRequest.getUnit();
        final Message message = unitRequest.getMessage();

        final CheckSave<Message> checkSave = answerSave.getCheckSave();
        if (checkNotNull(checkSave)) {
            return Uni.createFrom().voidItem()
                    .onItem().transformToUni(
                            v -> checkSave.check(message)
                                    .flatMap(
                                            unit -> {
                                                if (checkNotNull(unit)) {
                                                    return Uni.createFrom().item(UnitRequest.of(unit, message));
                                                }
                                                return getUnitRequestUni(answerSave, message);
                                            }
                                    )
                    );
        }

        return getUnitRequestUni(answerSave, message);
    }

    private Uni<UnitRequest<MainUnit, Message>> getUnitRequestUni(AnswerSave<Message, D> answerSave, Message message) {
        final String personId = message.getFromPersonId();
        final AnswerSavePreservable<D> preservable = answerSave.getPreservable();
        final PreservableData<D, Message> preservableData = answerSave.getPreservableData();
        final Pusher<D> pusher = answerSave.getPusher();
        return Uni.createFrom().voidItem()
                .onItem().transformToUni(
                        v -> {
                            if (checkNotNull(preservableData)) {
                                return preservableData.getData(message);
                            }
                            return Uni.createFrom().nullItem();
                        }
                ).onItem().transformToUni(
                        dataFromSave -> {
                            if (checkNotNull(dataFromSave)) {
                                return preservable.save(personId, answerSave.getKey(), dataFromSave);
                            }
                            return Uni.createFrom().nullItem();
                        }
                ).onItem().transformToUni(
                        v -> {
                            if (checkNotNull(pusher)) {
                                return preservable.push(personId, pusher);
                            }
                            return Uni.createFrom().nullItem();
                        }
                ).onItem().transform(
                        v -> UnitRequest.of(answerSave, message)
                );
    }

    @Override
    public String getUnitType() {
        return AnswerSave.TYPE;
    }

    @Override
    public Class<Message> getMessageType() {
        return Message.class;
    }

}
