package dev.struchkov.godfather.quarkus.core.action;

import dev.struchkov.godfather.main.domain.content.Mail;
import dev.struchkov.godfather.quarkus.context.service.Sending;
import dev.struchkov.godfather.quarkus.domain.unit.AnswerText;
import dev.struchkov.godfather.quarkus.domain.unit.MainUnit;
import dev.struchkov.godfather.quarkus.domain.unit.UnitRequest;
import dev.struchkov.godfather.quarkus.domain.unit.func.CallBackConsumer;
import io.smallrye.mutiny.Uni;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerText}.
 *
 * @author upagge [11/07/2019]
 */
public class AnswerTextAction implements ActionUnit<AnswerText<Mail>, Mail> {

    private final Sending sending;

    public AnswerTextAction(Sending sending) {
        this.sending = sending;
    }

    @Override
    public Uni<UnitRequest<MainUnit, Mail>> action(UnitRequest<AnswerText<Mail>, Mail> unitRequest) {
        final Mail message = unitRequest.getMessage();
        final AnswerText<Mail> unit = unitRequest.getUnit();

        return unit.getAnswer().processing(message)
                .onItem().ifNotNull().transformToUni(boxAnswer -> {
                    boxAnswer.setRecipientIfNull(message.getFromPersonId());
                    return sending.send(boxAnswer);
                })
                .onItem().ifNotNull().transformToUni(sentBox -> {
                    final CallBackConsumer callBack = unit.getCallBack();
                    if (checkNotNull(callBack)) {
                        return callBack.processing(sentBox);
                    }
                    return Uni.createFrom().voidItem();
                })
                .replaceWith(UnitRequest.of(unit, message));
    }

    @Override
    public String getUnitType() {
        return AnswerText.TYPE;
    }

    @Override
    public Class<Mail> getMessageType() {
        return Mail.class;
    }

}
