package dev.struchkov.godfather.quarkus.core.action;

import dev.struchkov.godfather.main.domain.content.ChatMail;
import dev.struchkov.godfather.quarkus.context.service.Sending;
import dev.struchkov.godfather.quarkus.domain.unit.AnswerText;
import dev.struchkov.godfather.quarkus.domain.unit.MainUnit;
import dev.struchkov.godfather.quarkus.domain.unit.UnitRequest;
import dev.struchkov.godfather.quarkus.domain.unit.func.CallBackConsumer;
import io.smallrye.mutiny.Uni;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerText}.
 *
 * @author upagge [11/07/2019]
 */
public class AnswerTextChatMailAction implements ActionUnit<AnswerText<ChatMail>, ChatMail> {

    private final Sending sending;

    public AnswerTextChatMailAction(Sending sending) {
        this.sending = sending;
    }

    @Override
    public Uni<UnitRequest<MainUnit, ChatMail>> action(UnitRequest<AnswerText<ChatMail>, ChatMail> unitRequest) {
        final ChatMail message = unitRequest.getMessage();
        final AnswerText<ChatMail> unit = unitRequest.getUnit();

        return unit.getAnswer().processing(message)
                .onItem().ifNotNull().transformToUni(boxAnswer -> {
                    boxAnswer.setRecipientIfNull(message.getChatId());
                    return sending.send(boxAnswer);
                })
                .onItem().ifNotNull().transformToUni(sentBox -> {
                    final CallBackConsumer callBack = unit.getCallBack();
                    if (checkNotNull(callBack)) {
                        return callBack.processing(sentBox);
                    }
                    return Uni.createFrom().voidItem();
                })
                .replaceWith(UnitRequest.of(unit, message));
    }

    @Override
    public String getUnitType() {
        return AnswerText.TYPE;
    }

    @Override
    public Class<ChatMail> getMessageType() {
        return ChatMail.class;
    }

}
