package dev.struchkov.godfather.quarkus.core.service;

import dev.struchkov.godfather.quarkus.context.repository.StorylineContext;
import dev.struchkov.haiti.utils.container.ContextKey;
import io.smallrye.mutiny.Uni;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Map;

import static dev.struchkov.haiti.context.exception.NotFoundException.notFoundException;
import static dev.struchkov.haiti.utils.Checker.checkNotNull;
import static dev.struchkov.haiti.utils.Inspector.isNotNull;

public class StorylineContextMapImpl implements StorylineContext {

    private final Map<String, Map<String, Object>> map = new HashMap<>();

    @Override
    public <T> Uni<Void> save(@NotNull String personId, @NotNull ContextKey<T> key, T objectForSave) {
        isNotNull(personId, key);
        map.computeIfAbsent(personId, k -> new HashMap<>()).put(key.getValue(), objectForSave);
        return Uni.createFrom().voidItem();
    }

    public <T> Uni<T> getByKey(@NotNull String personId, @NotNull ContextKey<T> key) {
        isNotNull(personId, key);
        if (map.containsKey(personId)) {
            final Map<String, Object> storage = map.get(personId);
            final T object = (T) storage.get(key.getValue());
            if (object != null && object.getClass().equals(key.getType())) {
                return Uni.createFrom().item(object);
            }
        }
        return Uni.createFrom().nullItem();
    }

    @Override
    public <T> Uni<T> removeKey(@NotNull String personId, @NotNull ContextKey<T> key) {
        isNotNull(personId, key);
        if (map.containsKey(personId)) {
            final Map<String, Object> storage = map.get(personId);
            final T object = (T) storage.get(key.getValue());
            if (checkNotNull(object) && object.getClass().equals(key.getType())) {
                storage.remove(key.getValue());
                return Uni.createFrom().item(object);
            }
        }
        return Uni.createFrom().nullItem();
    }

    @Override
    public <T> Uni<T> getByKeyOrThrow(@NotNull String personId, @NotNull ContextKey<T> key) {
        return getByKey(personId, key)
                .onItem().ifNull().failWith(notFoundException("Не найдено значение ключа {0}, для пользователя {1}", key.getValue(), personId));
    }

    @Override
    public Uni<Map<String, Object>> getAllSaveElement(@NotNull String personId) {
        isNotNull(personId);
        return Uni.createFrom().item(map.get(personId));
    }

    @Override
    public Uni<Void> removeAll(@NotNull String personId) {
        isNotNull(personId);
        map.remove(personId);
        return Uni.createFrom().voidItem();
    }

}
