package dev.struchkov.godfather.simple.core;

import dev.struchkov.godfather.main.domain.EventContainer;
import dev.struchkov.godfather.simple.context.service.EventDispatching;
import dev.struchkov.godfather.simple.context.service.EventHandler;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

public class EventDispatchingImpl implements EventDispatching {

    private final Map<Class, List<EventHandler>> eventHandlerMap;

    public EventDispatchingImpl(List<EventHandler> eventProviders) {
        this.eventHandlerMap = eventProviders.stream().collect(Collectors.groupingBy(EventHandler::getEventClass));
    }

    @Override
    public void dispatch(EventContainer event) {
        if (checkNotNull(event)) {
            final Optional<List<EventHandler>> optHandlers = getHandler(event.getType());
            if (optHandlers.isPresent()) {
                final List<EventHandler> eventHandlers = optHandlers.get();
                eventHandlers.forEach(eventHandler -> eventHandler.handle(event));
            }
        }
    }

    private Optional<List<EventHandler>> getHandler(Class type) {
        return Optional.ofNullable(eventHandlerMap.get(type));
    }

}
