package dev.struchkov.godfather.simple.core.action;

import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.simple.context.service.Sending;
import dev.struchkov.godfather.simple.domain.BoxAnswer;
import dev.struchkov.godfather.simple.domain.unit.AnswerCheck;
import dev.struchkov.godfather.simple.domain.unit.MainUnit;
import dev.struchkov.godfather.simple.domain.unit.UnitRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.Objects;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerCheck}.
 *
 * @author upagge [11/07/2019]
 */
@Slf4j
@RequiredArgsConstructor
public class AnswerCheckAction implements ActionUnit<AnswerCheck<Message>, Message> {

    private final Sending sending;

    @Override
    public UnitRequest<MainUnit, Message> action(UnitRequest<AnswerCheck<Message>, Message> unitRequest) {
        final AnswerCheck<Message> unit = unitRequest.getUnit();
        log.debug("Началась обработка юнита: {}.", unit.getName());

        final Message message = unitRequest.getMessage();

        MainUnit<Message> unitAnswer;
        if (unit.getCheck().checked(message)) {
            log.debug("Unit: {}. Проверка пройдена", unit.getName());
            final BoxAnswer answerIfTrue = unit.getIntermediateAnswerIfTrue();
            if (checkNotNull(answerIfTrue)) {
                answerIfTrue.setRecipientIfNull(message.getFromPersonId());
                log.trace("Будет отправлен промежуточный ответ: {}", answerIfTrue);
                sending.send(answerIfTrue);
            }
            unitAnswer = unit.getUnitTrue();
        } else {
            log.debug("Unit: {}. Проверка НЕ пройдена", unit.getName());
            final BoxAnswer answerIfFalse = unit.getIntermediateAnswerIfFalse();
            if (checkNotNull(answerIfFalse)) {
                answerIfFalse.setRecipientIfNull(message.getFromPersonId());
                log.trace("Будет отправлен промежуточный ответ: {}", answerIfFalse);
                sending.send(answerIfFalse);
            }
            unitAnswer = unit.getUnitFalse();
        }
        log.debug("Завершилась обработка unit: {}.", unit.getName());
        return UnitRequest.of(Objects.requireNonNullElse(unitAnswer, unit), message);
    }

    @Override
    public String getUnitType() {
        return AnswerCheck.TYPE;
    }

    @Override
    public Class<Message> getMessageType() {
        return Message.class;
    }

}
