package dev.struchkov.godfather.simple.core.action;

import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.simple.domain.unit.AnswerSave;
import dev.struchkov.godfather.simple.domain.unit.MainUnit;
import dev.struchkov.godfather.simple.domain.unit.UnitRequest;
import dev.struchkov.godfather.simple.domain.unit.func.CheckSave;
import dev.struchkov.godfather.simple.domain.unit.func.PreservableData;
import dev.struchkov.godfather.simple.domain.unit.func.Pusher;
import dev.struchkov.godfather.simple.domain.unit.func.preser.AnswerSavePreservable;
import lombok.extern.slf4j.Slf4j;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerSave}.
 *
 * @author upagge [11/07/2019]
 */
@Slf4j
public class AnswerSaveAction<D> implements ActionUnit<AnswerSave<Message, D>, Message> {

    @Override
    public UnitRequest<MainUnit, Message> action(UnitRequest<AnswerSave<Message, D>, Message> unitRequest) {
        final AnswerSave<Message, D> answerSave = unitRequest.getUnit();

        log.debug("Началась обработка юнита: {}.", answerSave.getName());

        final Message message = unitRequest.getMessage();

        final AnswerSavePreservable<D> preservable = answerSave.getPreservable();
        final String personId = message.getFromPersonId();

        final CheckSave<Message> checkSave = answerSave.getCheckSave();
        if (checkNotNull(checkSave)) {
            final MainUnit<Message> unit = checkSave.check(message);
            if (checkNotNull(unit)) {
                return UnitRequest.of(unit, message);
            }
        }

        final PreservableData<D, Message> preservableData = answerSave.getPreservableData();
        if (checkNotNull(preservableData)) {
            D data = preservableData.getData(message);
            if (checkNotNull(data)) {
                preservable.save(personId, answerSave.getKey(), data);
            }
        }

        final Pusher<D> pusher = answerSave.getPusher();
        if (checkNotNull(pusher)) {
            preservable.push(personId, pusher);
        }

        log.debug("Завершилась обработка unit: {}.", answerSave.getName());
        return UnitRequest.of(answerSave, message);
    }

    @Override
    public String getUnitType() {
        return AnswerSave.TYPE;
    }

    @Override
    public Class<Message> getMessageType() {
        return Message.class;
    }

}
