package dev.struchkov.godfather.simple.core.action;

import dev.struchkov.godfather.main.domain.content.ChatMail;
import dev.struchkov.godfather.simple.context.service.Sending;
import dev.struchkov.godfather.simple.domain.BoxAnswer;
import dev.struchkov.godfather.simple.domain.SentBox;
import dev.struchkov.godfather.simple.domain.unit.AnswerText;
import dev.struchkov.godfather.simple.domain.unit.MainUnit;
import dev.struchkov.godfather.simple.domain.unit.UnitRequest;
import dev.struchkov.godfather.simple.domain.unit.func.CallBackConsumer;
import dev.struchkov.godfather.simple.domain.unit.func.ProcessingData;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.Optional;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerText}.
 *
 * @author upagge [11/07/2019]
 */
@Slf4j
@RequiredArgsConstructor
public class AnswerTextChatMailAction implements ActionUnit<AnswerText<ChatMail>, ChatMail> {

    private final Sending sending;

    @Override
    public UnitRequest<MainUnit, ChatMail> action(UnitRequest<AnswerText<ChatMail>, ChatMail> unitRequest) {
        final ChatMail message = unitRequest.getMessage();
        final AnswerText<ChatMail> unit = unitRequest.getUnit();

        final ProcessingData<ChatMail> answerProcessing = unit.getAnswer();
        if (checkNotNull(answerProcessing)) {
            final Optional<BoxAnswer> optAnswer = answerProcessing.processing(message);
            if (optAnswer.isPresent()) {
                final BoxAnswer answer = optAnswer.get();

                answer.setRecipientIfNull(message.getChatId());

                final Optional<? extends SentBox> optSentBox = sending.send(answer);
                final CallBackConsumer callBack = unit.getCallBack();
                if (checkNotNull(callBack) && optAnswer.isPresent()) {
                    final SentBox sentBox = optSentBox.get();
                    callBack.processing(sentBox);
                }
            }
        }

        return UnitRequest.of(unit, message);
    }

    @Override
    public String getUnitType() {
        return AnswerText.TYPE;
    }

    @Override
    public Class<ChatMail> getMessageType() {
        return ChatMail.class;
    }

}
