package dev.struchkov.godfather.simple.core.action;

import dev.struchkov.godfather.main.domain.content.Mail;
import dev.struchkov.godfather.simple.context.service.Sending;
import dev.struchkov.godfather.simple.domain.BoxAnswer;
import dev.struchkov.godfather.simple.domain.SentBox;
import dev.struchkov.godfather.simple.domain.unit.AnswerText;
import dev.struchkov.godfather.simple.domain.unit.MainUnit;
import dev.struchkov.godfather.simple.domain.unit.UnitRequest;
import dev.struchkov.godfather.simple.domain.unit.func.CallBackConsumer;
import dev.struchkov.godfather.simple.domain.unit.func.ProcessingData;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.Optional;

import static dev.struchkov.haiti.utils.Checker.checkNotNull;

/**
 * Обработчик Unit-а {@link AnswerText}.
 *
 * @author upagge [11/07/2019]
 */
@Slf4j
@RequiredArgsConstructor
public class AnswerTextMailAction implements ActionUnit<AnswerText<Mail>, Mail> {

    private final Sending sending;

    @Override
    public UnitRequest<MainUnit, Mail> action(UnitRequest<AnswerText<Mail>, Mail> unitRequest) {
        final AnswerText<Mail> unit = unitRequest.getUnit();
        final Mail message = unitRequest.getMessage();

        log.debug("Началась обработка юнита: {}.", unit.getName());

        final ProcessingData<Mail> answerProcessing = unit.getAnswer();
        if (checkNotNull(answerProcessing)) {
            log.trace("Начало выполнения клиентской логики");
            final Optional<BoxAnswer> optAnswer = answerProcessing.processing(message);
            log.trace("Конец выполнения клиентской логики");
            if (optAnswer.isPresent()) {
                final BoxAnswer answer = optAnswer.get();
                answer.setRecipientIfNull(message.getFromPersonId());

                log.debug("AnswerText предоставил ответ для отправки.", answer);

                final Optional<? extends SentBox> optSentBox = sending.send(answer);
                final CallBackConsumer callBack = unit.getCallBack();
                if (checkNotNull(callBack) && optAnswer.isPresent()) {
                    final SentBox sentBox = optSentBox.get();
                    log.debug("Вызывается callBack юнита. Название юнита: {}", unit.getName());
                    callBack.processing(sentBox);
                }
            } else {
                log.debug("AnswerText не предоставил ответ для отправки. Название юнита: {}", unit.getName());
            }
        } else {
            log.warn("У AnswerText отсутствовал answer. Обработка не выполнялась. Название юнита: {}", unit.getName());
        }

        log.debug("Завершилась обработка unit: {}.", unit.getName());
        return UnitRequest.of(unit, message);
    }

    @Override
    public String getUnitType() {
        return AnswerText.TYPE;
    }

    @Override
    public Class<Mail> getMessageType() {
        return Mail.class;
    }

}
