package dev.struchkov.godfather.simple.core.action.cmd;

import dev.struchkov.godfather.main.domain.StorylineHistory;
import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.simple.core.action.ActionUnit;
import dev.struchkov.godfather.simple.core.service.StorylineService;
import dev.struchkov.godfather.simple.domain.unit.MainUnit;
import dev.struchkov.godfather.simple.domain.unit.UnitRequest;
import dev.struchkov.godfather.simple.domain.unit.cmd.RollBackCmd;

import static dev.struchkov.godfather.exception.RollBackException.rollBackException;

public class RollBackCmdAction implements ActionUnit<RollBackCmd<Message>, Message> {

    private final StorylineService<Message> storyLineService;

    public RollBackCmdAction(StorylineService<Message> storyLineService) {
        this.storyLineService = storyLineService;
    }

    @Override
    public UnitRequest<MainUnit, Message> action(UnitRequest<RollBackCmd<Message>, Message> unitRequest) {
        final RollBackCmd<Message> unit = unitRequest.getUnit();
        final Message message = unitRequest.getMessage();

        final int countToBack = unit.getCountBack();
        final String rollbackUnitName = unit.getRollbackUnitName();

        StorylineHistory history = rollbackUnitName != null
                ? storyLineService.replaceUserToBack(message.getFromPersonId(), rollbackUnitName).orElseThrow(rollBackException("Юнит для возвращения не был найден"))
                : storyLineService.replaceUserToBack(message.getFromPersonId(), countToBack).orElseThrow(rollBackException("Юнит для возвращения не был найден"));
        final String unitName = history.getUnitName();
        final MainUnit<Message> nextUnit = storyLineService.getUnitByName(unitName).orElse(unit);
        final Message oldMessage = history.getMessage();
        return UnitRequest.of(nextUnit, oldMessage);
    }

    @Override
    public String getUnitType() {
        return RollBackCmd.TYPE;
    }

    @Override
    public Class<Message> getMessageType() {
        return Message.class;
    }

}
