package dev.struchkov.godfather.simple.core.service;

import dev.struchkov.godfather.simple.context.repository.StorylineContext;
import dev.struchkov.haiti.utils.container.ContextKey;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import static dev.struchkov.haiti.context.exception.NotFoundException.notFoundException;
import static dev.struchkov.haiti.utils.Checker.checkNotNull;
import static dev.struchkov.haiti.utils.Inspector.isNotNull;

public class StorylineContextMapImpl implements StorylineContext {

    private final Map<String, Map<String, Object>> map = new HashMap<>();


    @Override
    public <T> void save(@NotNull String personId, @NotNull ContextKey<T> key, T objectForSave) {
        isNotNull(personId, key);
        map.computeIfAbsent(personId, k -> new HashMap<>()).put(key.getValue(), objectForSave);
    }

    @Override
    public <T> Optional<T> getByKey(@NotNull String personId, @NotNull ContextKey<T> key) {
        isNotNull(personId, key);
        if (map.containsKey(personId)) {
            final Map<String, Object> storage = map.get(personId);
            final T object = (T) storage.get(key.getValue());
            if (object != null && object.getClass().equals(key.getType())) {
                return Optional.of(object);
            }
        }
        return Optional.empty();
    }

    @Override
    public <T> Optional<T> removeKey(@NotNull String personId, @NotNull ContextKey<T> key) {
        isNotNull(personId, key);
        if (map.containsKey(personId)) {
            final Map<String, Object> storage = map.get(personId);
            final T object = (T) storage.get(key.getValue());
            if (checkNotNull(object) && object.getClass().equals(key.getType())) {
                storage.remove(key.getValue());
                return Optional.of(object);
            }
        }
        return Optional.empty();
    }

    @Override
    public <T> T getByKeyOrThrow(@NotNull String personId, @NotNull ContextKey<T> key) {
        return getByKey(personId, key).orElseThrow(notFoundException("Не найдено значение ключа {0}, для пользователя {1}", key.getValue(), personId));
    }

    @Override
    public Map<String, Object> getAllSaveElement(@NotNull String personId) {
        isNotNull(personId);
        return map.get(personId);
    }

    @Override
    public void removeAll(@NotNull String personId) {
        isNotNull(personId);
        map.remove(personId);
    }

}
