package dev.struchkov.godfather.quarkus.domain.unit;

import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.main.domain.unit.KeyWord;
import dev.struchkov.godfather.main.domain.unit.UnitActiveType;
import dev.struchkov.godfather.quarkus.domain.unit.func.UniPredicate;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;

import java.util.Collection;
import java.util.Optional;
import java.util.Set;
import java.util.regex.Pattern;

/**
 * Главный обработчик, от него наследуются все остальные Unit-ы.
 *
 * @author upagge [08/07/2019]
 */
@Getter
@Setter
@EqualsAndHashCode(onlyExplicitlyIncluded = true)
public abstract class MainUnit<M extends Message> {

    /**
     * Уникальное имя юнита
     */
    @EqualsAndHashCode.Include
    private String name;

    /**
     * Ключевые слова.
     */
    protected Set<KeyWord> triggerWords;

    /**
     * Точная фраза.
     */
    protected Set<String> phrases;

    /**
     * Триггеры на срабатывание юнита по регулярному выражению.
     */
    protected Set<Pattern> triggerPatterns;

    /**
     * Пользовательский триггер
     */
    protected UniPredicate<M> triggerCheck;

    /**
     * Значение минимального отношения количества найденных ключевых слов, к количеству ключевых слов Unit-а.
     */
    protected Integer matchThreshold;

    /**
     * Значение приоритета.
     */
    protected Integer priority;

    /**
     * Множество следующих Unit в сценарии.
     */
    protected Set<MainUnit<M>> nextUnits;

    /**
     * Описание юнита, что он делает. Никак не влияет на работу юнита и не участвует в ней. Возможно отображение этого текста в логах.
     */
    private final String description;

    /**
     * Тип Unit-а.
     */
    protected final String type;

    /**
     * Режим срабатывания Unit-а.
     */
    protected UnitActiveType activeType;

    private final boolean notSaveHistory;

    protected MainUnit(
            String name,
            Set<KeyWord> triggerWords,
            Set<String> phrases,
            Set<Pattern> triggerPatterns,
            UniPredicate<M> triggerCheck,
            Integer matchThreshold,
            Integer priority,
            Set<MainUnit<M>> nextUnits,
            String description,
            String type,
            UnitActiveType activeType,
            boolean notSaveHistory
    ) {
        this.name = name;
        this.triggerWords = triggerWords;
        this.phrases = phrases;
        this.triggerPatterns = triggerPatterns;
        this.triggerCheck = triggerCheck;
        this.matchThreshold = matchThreshold == null ? 10 : matchThreshold;
        this.priority = priority == null ? 10 : priority;
        this.nextUnits = nextUnits;
        this.description = description;
        this.type = type;
        this.activeType = Optional.ofNullable(activeType).orElse(UnitActiveType.DEFAULT);
        this.notSaveHistory = notSaveHistory;
    }

    public void addPhrase(String phrase) {
        phrases.add(phrase);
    }

    public void addPhrases(Collection<String> phrases) {
        this.phrases.addAll(phrases);
    }

}
