package dev.struchkov.godfather.quarkus.domain.unit;

import dev.struchkov.godfather.main.domain.content.Message;
import dev.struchkov.godfather.main.domain.unit.KeyWord;
import dev.struchkov.godfather.main.domain.unit.UnitActiveType;
import dev.struchkov.godfather.quarkus.domain.unit.func.UniPredicate;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

public class SwitchUnit<M extends Message> extends MainUnit<M> {

    public static final String TYPE = "SWITCH_UNIT";

    private SwitchUnit(Builder<M> builder) {
        super(
                builder.name,
                builder.triggerWords,
                builder.triggerPhrases,
                builder.triggerPatterns,
                builder.triggerCheck,
                builder.matchThreshold,
                builder.priority,
                builder.nextUnits,
                builder.description,
                TYPE,
                builder.activeType,
                true
        );
    }

    public static <M extends Message> SwitchUnit.Builder<M> builder() {
        return new SwitchUnit.Builder<>();
    }

    public static final class Builder<M extends Message> {

        private String name = UUID.randomUUID().toString();
        private String description;
        private Set<MainUnit<M>> nextUnits;

        private Set<KeyWord> triggerWords;
        private Set<String> triggerPhrases;
        private UniPredicate<M> triggerCheck;
        private Set<Pattern> triggerPatterns;

        private Integer matchThreshold;
        private Integer priority;

        private UnitActiveType activeType;

        private Builder() {
        }

        public SwitchUnit.Builder<M> name(String name) {
            this.name = name;
            return this;
        }

        public SwitchUnit.Builder<M> description(String description) {
            this.description = description;
            return this;
        }

        public SwitchUnit.Builder<M> triggerWords(Set<KeyWord> val) {
            if (triggerWords == null) {
                triggerWords = new HashSet<>();
            }
            triggerWords.addAll(val);
            return this;
        }

        public SwitchUnit.Builder<M> triggerWord(KeyWord val) {
            if (triggerWords == null) {
                triggerWords = new HashSet<>();
            }
            triggerWords.add(val);
            return this;
        }

        public SwitchUnit.Builder<M> triggerStringWords(Set<String> val) {
            if (triggerWords == null) {
                triggerWords = new HashSet<>();
            }
            triggerWords.addAll(val.stream().map(KeyWord::of).collect(Collectors.toSet()));
            return this;
        }

        public SwitchUnit.Builder<M> triggerWord(String val) {
            if (triggerWords == null) {
                triggerWords = new HashSet<>();
            }
            triggerWords.add(KeyWord.of(val));
            return this;
        }

        public SwitchUnit.Builder<M> triggerPhrase(String... val) {
            if (triggerPhrases == null) {
                triggerPhrases = new HashSet<>();
            }
            if (val.length == 1) {
                triggerPhrases.add(val[0]);
            } else {
                triggerPhrases.addAll(Set.of(val));
            }
            triggerPhrases.addAll(List.of(val));
            return this;
        }

        public SwitchUnit.Builder<M> triggerPattern(Pattern... val) {
            if (triggerPatterns == null) {
                triggerPatterns = new HashSet<>();
            }
            if (val.length == 1) {
                triggerPatterns.add(val[0]);
            } else {
                triggerPatterns.addAll(Set.of(val));
            }
            triggerPatterns.addAll(Set.of(val));
            return this;
        }

        public SwitchUnit.Builder<M> triggerCheck(UniPredicate<M> trigger) {
            triggerCheck = trigger;
            return this;
        }

        public SwitchUnit.Builder<M> matchThreshold(Integer val) {
            matchThreshold = val;
            return this;
        }

        public SwitchUnit.Builder<M> priority(Integer val) {
            priority = val;
            return this;
        }

        public SwitchUnit.Builder<M> next(MainUnit<M> val) {
            if (nextUnits == null) {
                nextUnits = new HashSet<>();
            }
            nextUnits.add(val);
            return this;
        }

        public SwitchUnit.Builder<M> activeType(UnitActiveType val) {
            activeType = val;
            return this;
        }

        public SwitchUnit<M> build() {
//            isNotNull(boxAnswer, UnitConfigException.unitConfigException("BoxAnswer обязательный параметр юнита"));
            return new SwitchUnit<>(this);
        }

    }

}
